@@ -5,11 +5,9 @@
 #' @details Run the shiny app by \code{results <- specify_graph()}. See detailed instructions in the vignette \code{browseVignettes("causaloptim")}.
 #' @author Michael C Sachs, Arvid Sjölander, Gustav Jonzon, Alexander Balke, Colorado Reed, and Erin Gabriel
 #'     Maintainer: Michael C Sachs <sachsmc at gmail.com>
-#' @references Sachs, M. C., Jonzon, G., Sjölander, A., & Gabriel, E. E. 2022, A general method for deriving tight symbolic bounds on causal effects. Journal of Computational and Graphical Statistics, In press. \url{https://www.tandfonline.com/doi/full/10.1080/10618600.2022.2071905} .
+#' @references Sachs, M. C., Jonzon, G., Sjölander, A., & Gabriel, E. E. (2023). A general method for deriving tight symbolic bounds on causal effects. Journal of Computational and Graphical Statistics, 32(2), 567-576. \url{https://www.tandfonline.com/doi/full/10.1080/10618600.2022.2071905} .
 #' 
-#' @docType package
 #' @seealso \code{browseVignettes('causaloptim')} \link{specify_graph}
 #' @name causaloptim-package
 #' @aliases causaloptim
-
-NULL
\ No newline at end of file
+"_PACKAGE"
\ No newline at end of file
